@@ -1,9 +1,26 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/common.c,v 1.16.2.1 2003/09/18 15:43:28 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
 /* common.c - common routines for the ldap client tools */
+/* $OpenLDAP: pkg/ldap/clients/tools/common.c,v 1.16.2.3 2003/12/01 00:58:22 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * Portions Copyright 2003 Kurt D. Zeilenga.
+ * Portions Copyright 2003 IBM Corporation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This file was initially created by Hallvard B. Furuseth based (in
+ * part) upon argument parsing code for individual tools located in
+ * this directory.   Additional contributors include:
+ *   Kurt D. Zeilenga (additional common argument and control support)
+ */
 
 #include "portable.h"
 
@@ -20,6 +37,7 @@
 #include "lutil_ldap.h"
 #include "ldap_defaults.h"
 #include "ldap_pvt.h"
+#include "lber_pvt.h"
 
 #include "common.h"
 
@@ -762,8 +780,8 @@ tool_server_controls( LDAP *ld, LDAPControl *extra_c, int count )
 	}
 
 	if ( assertctl ) {
-		char berbuf[LBER_ELEMENT_SIZEOF];
-		BerElement *ber = (BerElement *)berbuf;
+		BerElementBuffer berbuf;
+		BerElement *ber = (BerElement *)&berbuf;
 		
 		if( assertion == NULL || *assertion == '\0' ) {
 			fprintf( stderr, "Assertion=<empty>\n" );
@@ -820,7 +838,7 @@ tool_server_controls( LDAP *ld, LDAPControl *extra_c, int count )
 	if ( preread ) {
 		char berbuf[LBER_ELEMENT_SIZEOF];
 		BerElement *ber = (BerElement *)berbuf;
-		char **attrs;
+		char **attrs = NULL;
 
 		if( preread_attrs ) {
 			attrs = ldap_str2charray( preread_attrs, "," );
@@ -850,7 +868,7 @@ tool_server_controls( LDAP *ld, LDAPControl *extra_c, int count )
 	if ( postread ) {
 		char berbuf[LBER_ELEMENT_SIZEOF];
 		BerElement *ber = (BerElement *)berbuf;
-		char **attrs;
+		char **attrs = NULL;
 
 		if( postread_attrs ) {
 			attrs = ldap_str2charray( postread_attrs, "," );
