@@ -1,16 +1,18 @@
-/* $OpenLDAP: pkg/ldap/include/ldap.h,v 1.208.2.3 2003/09/18 15:43:31 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
+/* $OpenLDAP: pkg/ldap/include/ldap.h,v 1.208.2.5 2003/12/01 00:58:24 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ * 
+ * Copyright 1998-2003 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted only as authorized by the OpenLDAP
- * Public License.  A copy of this license is available at
- * http://www.OpenLDAP.org/license.html or in file LICENSE in the
- * top-level directory of the distribution.
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -206,14 +208,18 @@ typedef struct ldapcontrol {
 #define LDAP_CONTROL_SYNC_DONE	"1.3.6.1.4.1.4203.666.5.8"
 #define LDAP_SYNC_INFO			"1.3.6.1.4.1.4203.666.10.2"
 
-#define LDAP_SYNC_NEW_COOKIE		0
-#define LDAP_SYNC_STATE_MODE_DONE	1
-#define	LDAP_SYNC_LOG_MODE_DONE		2
-#define LDAP_SYNC_REFRESH_DONE		3
+#define LDAP_SYNC_REFRESH_PRESENTS	0
+#define LDAP_SYNC_REFRESH_DELETES   1
 
-#define LDAP_SYNC_STATE_MODE		0
-#define LDAP_SYNC_LOG_MODE			1
-#define LDAP_SYNC_PERSIST_MODE		2
+#define LDAP_TAG_SYNC_NEW_COOKIE		((ber_tag_t) 0x80U)
+#define LDAP_TAG_SYNC_REFRESH_DELETE	((ber_tag_t) 0xa1U)
+#define LDAP_TAG_SYNC_REFRESH_PRESENT	((ber_tag_t) 0xa2U)
+#define	LDAP_TAG_SYNC_ID_SET			((ber_tag_t) 0xa3U)
+
+#define LDAP_TAG_SYNC_COOKIE	((ber_tag_t) 0x04U)
+#define LDAP_TAG_REFRESHDELETES	((ber_tag_t) 0x01U)
+#define LDAP_TAG_REFRESHDONE	((ber_tag_t) 0x01U)
+#define LDAP_TAG_RELOAD_HINT	((ber_tag_t) 0x01U)
 
 #define LDAP_SYNC_PRESENT		0
 #define LDAP_SYNC_ADD			1
@@ -293,9 +299,6 @@ typedef struct ldapcontrol {
 
 #define LDAP_TAG_SASL_RES_CREDS	((ber_tag_t) 0x87U)	/* context specific + primitive */
 
-#define LDAP_SYNC_TAG_COOKIE	((ber_tag_t) 0x04U)	/* octet string */
-
-
 /* possible operations a client can invoke */
 #define LDAP_REQ_BIND		((ber_tag_t) 0x60U)	/* application + constructed */
 #define LDAP_REQ_UNBIND		((ber_tag_t) 0x42U)	/* application + primitive   */
