@@ -1,21 +1,25 @@
-/* $OpenLDAP: pkg/ldap/include/ldbm.h,v 1.39.2.1 2003/10/06 14:57:28 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
+/* $OpenLDAP: pkg/ldap/include/ldbm.h,v 1.39.2.3 2003/12/01 00:58:24 kurt Exp $ */
+/* ldbm.h - ldap dbm compatibility routine header file */
+/* $OpenLDAP: pkg/ldap/include/ldbm.h,v 1.39.2.3 2003/12/01 00:58:24 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted only as authorized by the OpenLDAP
- * Public License.  A copy of this license is available at
- * http://www.OpenLDAP.org/license.html or in file LICENSE in the
- * top-level directory of the distribution.
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* ldbm.h - ldap dbm compatibility routine header file */
 
 #ifndef _LDBM_H_
 #define _LDBM_H_
 
 #include <ldap_cdefs.h>
-#include <string.h>
+#include <ac/string.h>
 
 /* dummy DB_ENV for non Berkeley DB */
 #if !defined( LDBM_USE_DBBTREE ) && !defined( LDBM_USE_DBHASH )
