@@ -1,14 +1,21 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.77.2.1 2002/04/15 15:21:48 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.91.2.1 2003/12/01 00:58:26 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*  Portions
- *  Copyright (c) 1995 Regents of the University of Michigan.
- *  All rights reserved.
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
  */
-/*---
- * This notice applies to changes, created by or for Novell, Inc.,
+/* This notice applies to changes, created by or for Novell, Inc.,
  * to preexisting works for which notices appear elsewhere in this file.
  *
  * Copyright (C) 1999, 2000 Novell, Inc. All Rights Reserved.
@@ -25,6 +32,10 @@
  * Modification to OpenLDAP source by Novell, Inc.
  * April 2000 sfs  Added code to chase V3 referrals
  *  request.c - sending of ldap requests; handling of referrals
+ *---
+ * Note: A verbatim copy of version 2.0.1 of the OpenLDAP Public License 
+ * can be found in the file "build/LICENSE-2.0.1" in this distribution
+ * of OpenLDAP Software.
  */
 
 #include "portable.h"
@@ -78,13 +89,14 @@ ldap_send_initial_request(
 	LDAP *ld,
 	ber_tag_t msgtype,
 	const char *dn,
-	BerElement *ber )
+	BerElement *ber,
+	ber_int_t msgid)
 {
 	LDAPURLDesc	*servers;
 	int rc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_send_initial_request\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_send_initial_request\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_send_initial_request\n", 0, 0, 0 );
 #endif
@@ -99,8 +111,9 @@ ldap_send_initial_request(
 		}
 
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
-			"ldap_send_initial_request: ldap_open_defconn: successful\n" ));
+		LDAP_LOG ( OPERATION, DETAIL1, 
+			"ldap_send_initial_request: ldap_open_defconn: successful\n" , 
+			0, 0, 0);
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_open_defconn: successful\n",
@@ -128,14 +141,53 @@ ldap_send_initial_request(
 			return LDAP_PARAM_ERROR;
 	}
 #endif
-	rc = ldap_send_server_request( ld, ber, ld->ld_msgid, NULL,
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
+	rc = ldap_send_server_request( ld, ber, msgid, NULL,
 									servers, NULL, NULL );
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
 	if (servers)
 		ldap_free_urllist(servers);
 	return(rc);
 }
 
 
+int
+ldap_int_flush_request(
+	LDAP *ld,
+	LDAPRequest *lr
+)
+{
+	LDAPConn *lc = lr->lr_conn;
+
+	if ( ber_flush( lc->lconn_sb, lr->lr_ber, 0 ) != 0 ) {
+		if ( errno == EAGAIN ) {
+			/* need to continue write later */
+			lr->lr_status = LDAP_REQST_WRITING;
+			ldap_mark_select_write( ld, lc->lconn_sb );
+			ld->ld_errno = LDAP_BUSY;
+			return -2;
+		} else {
+			ld->ld_errno = LDAP_SERVER_DOWN;
+			ldap_free_request( ld, lr );
+			ldap_free_connection( ld, lc, 0, 0 );
+			return( -1 );
+		}
+	} else {
+		if ( lr->lr_parent == NULL ) {
+			lr->lr_ber->ber_end = lr->lr_ber->ber_ptr;
+			lr->lr_ber->ber_ptr = lr->lr_ber->ber_buf;
+		}
+		lr->lr_status = LDAP_REQST_INPROGRESS;
+
+		/* sent -- waiting for a response */
+		ldap_mark_select_read( ld, lc->lconn_sb );
+	}
+	return 0;
+}
 
 int
 ldap_send_server_request(
@@ -148,10 +200,10 @@ ldap_send_server_request(
 	LDAPreqinfo *bind )
 {
 	LDAPRequest	*lr;
-	int incparent;
+	int incparent, rc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_send_server_request\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_send_server_request\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_send_server_request\n", 0, 0, 0 );
 #endif
@@ -188,6 +240,20 @@ ldap_send_server_request(
 	}
 
 	use_connection( ld, lc );
+
+	/* If we still have an incomplete write, try to finish it before
+	 * dealing with the new request. If we don't finish here, return
+	 * LDAP_BUSY and let the caller retry later. We only allow a single
+	 * request to be in WRITING state.
+	 */
+	rc = 0;
+	if ( ld->ld_requests &&
+		ld->ld_requests->lr_status == LDAP_REQST_WRITING &&
+		ldap_int_flush_request( ld, ld->ld_requests ) < 0 ) {
+		rc = -1;
+	}
+	if ( rc ) return rc;
+
 	if (( lr = (LDAPRequest *)LDAP_CALLOC( 1, sizeof( LDAPRequest ))) ==
 	    NULL ) {
 		ld->ld_errno = LDAP_NO_MEMORY;
@@ -224,33 +290,11 @@ ldap_send_server_request(
 	ld->ld_requests = lr;
 	lr->lr_prev = NULL;
 
-	if ( ber_flush( lc->lconn_sb, ber, 0 ) != 0 ) {
-#ifdef notyet
-		if ( errno == EWOULDBLOCK ) {
-			/* need to continue write later */
-			lr->lr_status = LDAP_REQST_WRITING;
-			ldap_mark_select_write( ld, lc->lconn_sb );
-		} else {
-#else /* notyet */
-			ld->ld_errno = LDAP_SERVER_DOWN;
-			ldap_free_request( ld, lr );
-			ldap_free_connection( ld, lc, 0, 0 );
-			return( -1 );
-#endif /* notyet */
-#ifdef notyet
-		}
-#endif /* notyet */
-	} else {
-		if ( parentreq == NULL ) {
-			ber->ber_end = ber->ber_ptr;
-			ber->ber_ptr = ber->ber_buf;
-		}
-
-		/* sent -- waiting for a response */
-		ldap_mark_select_read( ld, lc->lconn_sb );
+	ld->ld_errno = LDAP_SUCCESS;
+	if ( ldap_int_flush_request( ld, lr ) == -1 ) {
+		msgid = -1;
 	}
 
-	ld->ld_errno = LDAP_SUCCESS;
 	return( msgid );
 }
 
@@ -263,7 +307,7 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 	Sockbuf		*sb = NULL;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_new_connection\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_new_connection\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_new_connection\n", 0, 0, 0 );
 #endif
@@ -329,14 +373,23 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 				ld->ld_defconn = lc;
 
 #ifdef NEW_LOGGING
-				LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
-					"ldap_new_connection: Call application rebind_proc\n" ));
+				LDAP_LOG ( OPERATION, DETAIL1, 
+					"ldap_new_connection: Call application rebind_proc\n", 
+					0, 0, 0 );
 #else
 				Debug( LDAP_DEBUG_TRACE, "Call application rebind_proc\n", 0, 0, 0);
+#endif
+#ifdef LDAP_R_COMPILE
+		ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+		ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
 #endif
 				err = (*ld->ld_rebind_proc)( ld,
 					bind->ri_url, bind->ri_request, bind->ri_msgid,
 					ld->ld_rebind_params );
+#ifdef LDAP_R_COMPILE
+		ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
+		ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
 
 				ld->ld_defconn = savedefconn;
 				--lc->lconn_refcnt;
@@ -354,14 +407,23 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 			ld->ld_defconn = lc;
 
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
-				"ldap_new_connection: anonymous rebind via ldap_bind_s\n" ));
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"ldap_new_connection: anonymous rebind via ldap_bind_s\n", 
+				0, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "anonymous rebind via ldap_bind_s\n", 0, 0, 0);
+#endif
+#ifdef LDAP_R_COMPILE
+			ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+			ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
 #endif
 			if ( ldap_bind_s( ld, "", "", LDAP_AUTH_SIMPLE ) != LDAP_SUCCESS ) {
 				err = -1;
 			}
+#ifdef LDAP_R_COMPILE
+			ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
+			ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
 			ld->ld_defconn = savedefconn;
 			--lc->lconn_refcnt;
 
@@ -422,7 +484,7 @@ ldap_free_connection( LDAP *ld, LDAPConn *lc, int force, int unbind )
 	LDAPConn	*tmplc, *prevlc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_free_connection\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_free_connection\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_free_connection\n", 0, 0, 0 );
 #endif
@@ -472,8 +534,8 @@ ldap_free_connection( LDAP *ld, LDAPConn *lc, int force, int unbind )
 		}
 		LDAP_FREE( lc );
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_RESULTS, 
-			"ldap_free_connection: actually freed\n" ));
+		LDAP_LOG ( OPERATION, RESULTS, 
+			"ldap_free_connection: actually freed\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE, "ldap_free_connection: actually freed\n",
 		    0, 0, 0 );
@@ -481,8 +543,8 @@ ldap_free_connection( LDAP *ld, LDAPConn *lc, int force, int unbind )
 	} else {
 		lc->lconn_lastused = time( NULL );
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_RESULTS, 
-			"ldap_free_connection: refcnt %d\n", lc->lconn_refcnt ));
+		LDAP_LOG ( OPERATION, RESULTS, "ldap_free_connection: refcnt %d\n", 
+			lc->lconn_refcnt, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE, "ldap_free_connection: refcnt %d\n",
 		    lc->lconn_refcnt, 0, 0 );
@@ -542,6 +604,9 @@ ldap_dump_requests_and_responses( LDAP *ld )
 	LDAPRequest	*lr;
 	LDAPMessage	*lm, *l;
 
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
 	fprintf( stderr, "** Outstanding Requests:\n" );
 	if (( lr = ld->ld_requests ) == NULL ) {
 		fprintf( stderr, "   Empty\n" );
@@ -557,7 +622,9 @@ ldap_dump_requests_and_responses( LDAP *ld )
 	    fprintf( stderr, "   outstanding referrals %d, parent count %d\n",
 		    lr->lr_outrefcnt, lr->lr_parentcnt );
 	}
-
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
 	fprintf( stderr, "** Response Queue:\n" );
 	if (( lm = ld->ld_responses ) == NULL ) {
 		fprintf( stderr, "   Empty\n" );
@@ -612,23 +679,22 @@ ldap_free_request( LDAP *ld, LDAPRequest *lr )
 	LDAPRequest     **ttmplr;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ARGS, 
-		"ldap_free_request (origid %d, msgid %d)\n",
-		lr->lr_origid, lr->lr_msgid ));
+	LDAP_LOG ( OPERATION, ARGS, "ldap_free_request (origid %d, msgid %d)\n",
+		lr->lr_origid, lr->lr_msgid, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_free_request (origid %d, msgid %d)\n",
 		lr->lr_origid, lr->lr_msgid, 0 );
 #endif
 
+	/* free all referrals (child requests) */
+	while ( lr->lr_child )
+		ldap_free_request( ld, lr->lr_child );
+
 	if ( lr->lr_parent != NULL ) {
 		--lr->lr_parent->lr_outrefcnt;
 		for ( ttmplr = &lr->lr_parent->lr_child; *ttmplr && *ttmplr != lr; ttmplr = &(*ttmplr)->lr_refnext ); 
 		if ( *ttmplr == lr )  
 			*ttmplr = lr->lr_refnext;
-	} else {
-		/* free all referrals (child requests) */
-		while ( lr->lr_child )
-			ldap_free_request( ld, lr->lr_child );
 	}
 	ldap_free_request_int( ld, lr );
 }
@@ -658,14 +724,14 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 	BerElement	*ber;
 	char		**refarray = NULL;
 	LDAPConn	*lc;
-	int			 rc, count, i, j;
+	int			 rc, count, i, j, id;
 	LDAPreqinfo  rinfo;
 
 	ld->ld_errno = LDAP_SUCCESS;	/* optimistic */
 	*hadrefp = 0;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_chase_v3referrals\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_chase_v3referrals\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_chase_v3referrals\n", 0, 0, 0 );
 #endif
@@ -682,9 +748,9 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 	/* Check for hop limit exceeded */
 	if ( lr->lr_parentcnt >= ld->ld_refhoplimit ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
+		LDAP_LOG ( OPERATION, DETAIL1, 
 			"ldap_chase_v3referrals: more than %d referral hops (dropping)\n",
-			ld->ld_refhoplimit ));
+			ld->ld_refhoplimit, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "more than %d referral hops (dropping)\n", ld->ld_refhoplimit, 0, 0 );
@@ -737,9 +803,9 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 				 * for the same server with different contexts.
 				 */
 #ifdef NEW_LOGGING
-				LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
+				LDAP_LOG ( OPERATION, DETAIL1, 
 					"ldap_chase_v3referrals: queue referral \"%s\"\n",
-					refarray[i] ));
+					refarray[i], 0, 0 );
 #else
 				Debug( LDAP_DEBUG_TRACE,
 					"ldap_chase_v3referrals: queue referral \"%s\"\n",
@@ -797,7 +863,8 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 			srv->lud_dn = LDAP_STRDUP( "" );
 		}
 
-		ber = re_encode_request( ld, origreq->lr_ber, ++ld->ld_msgid,
+		LDAP_NEXT_MSGID( ld, id );
+		ber = re_encode_request( ld, origreq->lr_ber, id,
 			sref, srv, &rinfo.ri_request );
 
 		if( ber == NULL ) {
@@ -807,9 +874,9 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 		}
 
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
+		LDAP_LOG ( OPERATION, DETAIL1, 
 			"ldap_chase_v3referrals: msgid %d, url \"%s\"\n",
-			lr->lr_msgid, refarray[i] ));
+			lr->lr_msgid, refarray[i], 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chase_v3referral: msgid %d, url \"%s\"\n",
@@ -819,13 +886,20 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 		/* Send the new request to the server - may require a bind */
 		rinfo.ri_msgid = origreq->lr_origid;
 		rinfo.ri_url = refarray[i];
-		if ( (rc = ldap_send_server_request( ld, ber, ld->ld_msgid,
-		    	origreq, srv, NULL, &rinfo )) < 0 ) {
+#ifdef LDAP_R_COMPILE
+		ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
+		rc = ldap_send_server_request( ld, ber, id,
+		    	origreq, srv, NULL, &rinfo );
+#ifdef LDAP_R_COMPILE
+		ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
+		if ( rc < 0 ) {
 			/* Failure, try next referral in the list */
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "request", LDAP_LEVEL_ERR, 
+			LDAP_LOG ( OPERATION, ERR, 
 			"ldap_chase_v3referrals: Unable to chase referral \"%s\" (%s)\n",
-			refarray[i], ldap_err2string( ld->ld_errno ) ));
+			refarray[i], ldap_err2string( ld->ld_errno ), 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "Unable to chase referral \"%s\" (%s)\n", 
 				refarray[i], ldap_err2string( ld->ld_errno ), 0);
@@ -899,7 +973,7 @@ ldap_chase_referrals( LDAP *ld,
 	int sref,
 	int *hadrefp )
 {
-	int		rc, count;
+	int		rc, count, id;
 	unsigned	len;
 	char		*p, *ref, *unfollowed;
 	LDAPRequest	*origreq;
@@ -908,7 +982,7 @@ ldap_chase_referrals( LDAP *ld,
 	LDAPreqinfo  rinfo;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, "ldap_chase_referrals\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_chase_referrals\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_chase_referrals\n", 0, 0, 0 );
 #endif
@@ -935,9 +1009,9 @@ ldap_chase_referrals( LDAP *ld,
 
 	if ( lr->lr_parentcnt >= ld->ld_refhoplimit ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_ENTRY, 
+		LDAP_LOG ( OPERATION, ENTRY, 
 			"ldap_chase_referrals: more than %d referral hops (dropping)\n",
-			ld->ld_refhoplimit ));
+			ld->ld_refhoplimit, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "more than %d referral hops (dropping)\n",
@@ -968,9 +1042,9 @@ ldap_chase_referrals( LDAP *ld,
 
 		if ( rc != LDAP_URL_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
+			LDAP_LOG ( OPERATION, DETAIL1, 
 				"ldap_chase_referrals: ignoring unknown referral <%s>\n",
-				ref ));
+				ref, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 			    "ignoring unknown referral <%s>\n", ref, 0, 0 );
@@ -986,8 +1060,8 @@ ldap_chase_referrals( LDAP *ld,
 		}
 
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1, 
-			"ldap_chase_referrals: chasing LDAP referral <%s>\n", ref ));
+		LDAP_LOG ( OPERATION, DETAIL1, 
+			"ldap_chase_referrals: chasing LDAP referral <%s>\n", ref, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 		    "chasing LDAP referral: <%s>\n", ref, 0, 0 );
@@ -995,8 +1069,9 @@ ldap_chase_referrals( LDAP *ld,
 
 		*hadrefp = 1;
 
+		LDAP_NEXT_MSGID( ld, id );
 		ber = re_encode_request( ld, origreq->lr_ber,
-		    ++ld->ld_msgid, sref, srv, &rinfo.ri_request );
+		    id, sref, srv, &rinfo.ri_request );
 
 		if( ber == NULL ) {
 			return -1 ;
@@ -1007,8 +1082,14 @@ ldap_chase_referrals( LDAP *ld,
 
 		rinfo.ri_msgid = origreq->lr_origid;
 
-		rc = ldap_send_server_request( ld, ber, ld->ld_msgid,
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
+		rc = ldap_send_server_request( ld, ber, id,
 		    lr, srv, NULL, &rinfo );
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
 
 		LDAP_FREE( rinfo.ri_url );
 
@@ -1016,9 +1097,9 @@ ldap_chase_referrals( LDAP *ld,
 			++count;
 		} else {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "request", LDAP_LEVEL_ERR, 
+			LDAP_LOG ( OPERATION, ERR, 
 				"ldap_chase_referrals: Unable to chase referral <%s>\n", 
-				ldap_err2string( ld->ld_errno) ));
+				ldap_err2string( ld->ld_errno), 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 			    "Unable to chase referral (%s)\n", 
@@ -1091,10 +1172,10 @@ re_encode_request( LDAP *ld,
 	char		*dn;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "request", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"re_encode_request: new msgid %ld, new dn <%s>\n",
 		(long) msgid, 
-		( srv == NULL || srv->lud_dn == NULL ) ? "NONE" : srv->lud_dn ));
+		( srv == NULL || srv->lud_dn == NULL ) ? "NONE" : srv->lud_dn, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 	    "re_encode_request: new msgid %ld, new dn <%s>\n",
@@ -1192,8 +1273,8 @@ re_encode_request( LDAP *ld,
 #ifdef LDAP_DEBUG
 	if ( ldap_debug & LDAP_DEBUG_PACKETS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "request", LDAP_LEVEL_DETAIL1,
-			"re_encode_request: new request is:\n" ));
+		LDAP_LOG ( OPERATION, DETAIL1, 
+			"re_encode_request: new request is:\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "re_encode_request new request is:\n",
 		    0, 0, 0 );
@@ -1212,6 +1293,9 @@ ldap_find_request_by_msgid( LDAP *ld, ber_int_t msgid )
 {
 	LDAPRequest	*lr;
 
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
 	for ( lr = ld->ld_requests; lr != NULL; lr = lr->lr_next ) {
 		if( lr->lr_status == LDAP_REQST_COMPLETED ) {
 			continue;	/* Skip completed requests */
@@ -1220,6 +1304,9 @@ ldap_find_request_by_msgid( LDAP *ld, ber_int_t msgid )
 			break;
 		}
 	}
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
 
 	return( lr );
 }
