@@ -1,14 +1,24 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.64.2.1 2002/04/15 15:21:48 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* LIBLDAP url.c -- LDAP URL (RFC 2255) related routines */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.74.2.1 2003/12/01 00:58:27 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*  Portions
- *  Copyright (c) 1996 Regents of the University of Michigan.
- *  All rights reserved.
- *
- *  LIBLDAP url.c -- LDAP URL (RFC 2255) related routines
- *
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
+ * All rights reserved.
+ */
+
+
+/*
  *  LDAP URLs look like this:
  *    ldap[is]://host:port[/[dn[?[attributes][?[scope][?[filter][?exts]]]]]]
  *
@@ -34,7 +44,6 @@
 
 #include "ldap-int.h"
 
-
 /* local functions */
 static const char* skip_url_prefix LDAP_P((
 	const char *url,
@@ -356,7 +365,7 @@ char * ldap_url_desc2str( LDAPURLDesc *u )
 	};
 
 	if( u->lud_port ) {
-		len+=6;
+		len += sizeof(":65535") - 1;
 	}
 
 	if( u->lud_host ) {
@@ -446,7 +455,7 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	 * the options and cause infinite recursion
 	 */
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "url", LDAP_LEVEL_ENTRY, "ldap_url_parse_ext(%s)\n", url_in ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_url_parse_ext(%s)\n", url_in, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_url_parse_ext(%s)\n", url_in, 0, 0 );
 #endif
@@ -881,17 +890,23 @@ ldap_url_duplist (LDAPURLDesc *ludlist)
 
 int
 ldap_url_parselist (LDAPURLDesc **ludlist, const char *url )
+{
+	return ldap_url_parselist_ext( ludlist, url, ", " );
+}
+
+int
+ldap_url_parselist_ext (LDAPURLDesc **ludlist, const char *url, const char *sep )
 {
 	int i, rc;
 	LDAPURLDesc *ludp;
 	char **urls;
 
+	assert( ludlist != NULL );
+	assert( url != NULL );
+
 	*ludlist = NULL;
 
-	if (url == NULL)
-		return LDAP_PARAM_ERROR;
-
-	urls = ldap_str2charray(url, ", ");
+	urls = ldap_str2charray(url, sep);
 	if (urls == NULL)
 		return LDAP_NO_MEMORY;
 
@@ -923,10 +938,10 @@ ldap_url_parsehosts(
 	LDAPURLDesc *ludp;
 	char **specs, *p;
 
-	*ludlist = NULL;
+	assert( ludlist != NULL );
+	assert( hosts != NULL );
 
-	if (hosts == NULL)
-		return LDAP_PARAM_ERROR;
+	*ludlist = NULL;
 
 	specs = ldap_str2charray(hosts, ", ");
 	if (specs == NULL)
@@ -1043,9 +1058,13 @@ ldap_url_list2urls(
 	/* figure out how big the string is */
 	size = 1;	/* nul-term */
 	for (ludp = ludlist; ludp != NULL; ludp = ludp->lud_next) {
-		size += strlen(ludp->lud_scheme) + strlen(ludp->lud_host);
-		if (strchr(ludp->lud_host, ':'))        /* will add [ ] below */
-			size += 2;
+		size += strlen(ludp->lud_scheme);
+		if ( ludp->lud_host ) {
+			size += strlen(ludp->lud_host);
+			/* will add [ ] below */
+			if (strchr(ludp->lud_host, ':'))
+				size += 2;
+		}
 		size += sizeof(":/// ");
 
 		if (ludp->lud_port != 0) {
@@ -1060,9 +1079,11 @@ ldap_url_list2urls(
 
 	p = s;
 	for (ludp = ludlist; ludp != NULL; ludp = ludp->lud_next) {
-		p += sprintf(p,
-			     strchr(ludp->lud_host, ':') ? "%s://[%s]" : "%s://%s",
-			     ludp->lud_scheme, ludp->lud_host);
+		p += sprintf(p, "%s://", ludp->lud_scheme);
+		if ( ludp->lud_host ) {
+			p += sprintf(p, strchr(ludp->lud_host, ':') 
+					? "[%s]" : "%s", ludp->lud_host);
+		}
 		if (ludp->lud_port != 0)
 			p += sprintf(p, ":%d", ludp->lud_port);
 		*p++ = '/';
@@ -1138,12 +1159,14 @@ ldap_pvt_hex_unescape( char *s )
 
 	for ( p = s; *s != '\0'; ++s ) {
 		if ( *s == '%' ) {
-			if ( *++s != '\0' ) {
-				*p = ldap_int_unhex( *s ) << 4;
+			if ( *++s == '\0' ) {
+				break;
 			}
-			if ( *++s != '\0' ) {
-				*p++ += ldap_int_unhex( *s );
+			*p = ldap_int_unhex( *s ) << 4;
+			if ( *++s == '\0' ) {
+				break;
 			}
+			*p++ += ldap_int_unhex( *s );
 		} else {
 			*p++ = *s;
 		}
