@@ -1,16 +1,19 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap_r/thr_stub.c,v 1.15 2002/01/04 20:17:40 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, Redwood City, California, USA
+/* thr_stub.c - stubs for the threads */
+/* $OpenLDAP: pkg/ldap/libraries/libldap_r/thr_stub.c,v 1.20.2.1 2003/12/01 00:58:27 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
-/* thr_stub.c - stubs for the threads */
-
 #include "portable.h"
 
 #if defined( NO_THREADS )
@@ -154,9 +157,9 @@ ldap_pvt_thread_pool_init (
 int
 ldap_pvt_thread_pool_submit (
 	ldap_pvt_thread_pool_t *pool,
-	void *(*start_routine)( void * ), void *arg )
+	ldap_pvt_thread_start_t *start_routine, void *arg )
 {
-	(start_routine)(arg);
+	(start_routine)(NULL, arg);
 	return(0);
 }
 
@@ -180,4 +183,27 @@ ldap_pvt_thread_pool_destroy (
 	return(0);
 }
 
+int ldap_pvt_thread_pool_getkey (
+	void *ctx, void *key, void **data, ldap_pvt_thread_pool_keyfree_t **kfree )
+{
+	return(0);
+}
+
+int ldap_pvt_thread_pool_setkey (
+	void *ctx, void *key, void *data, ldap_pvt_thread_pool_keyfree_t *kfree )
+{
+	return(0);
+}
+
+void *ldap_pvt_thread_pool_context( )
+{
+	return(NULL);
+}
+
+ldap_pvt_thread_t
+ldap_pvt_thread_self( void )
+{
+	return(0);
+}
+
 #endif /* NO_THREADS */
