@@ -1,7 +1,22 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/entropy.c,v 1.21.2.1 2002/04/15 15:21:51 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* entropy.c -- routines for providing pseudo-random data */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/entropy.c,v 1.25.2.1 2003/12/01 00:58:29 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1999-2003 The OpenLDAP Foundation.
+ * Portions Copyright 1999-2003 Kurt D. Zeilenga.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* This work was initially developed by Kurt D. Zeilenga for
+ * inclusion in OpenLDAP Software based, in part, on publically
+ * available works (as noted below).
  */
 
 #include "portable.h"
@@ -36,21 +51,27 @@ int lutil_entropy( unsigned char *buf, ber_len_t nbytes )
 	if( nbytes == 0 ) return 0;
 
 #ifdef URANDOM_DEVICE
+#define URANDOM_NREADS 4
 	/* Linux and *BSD offer a urandom device */
 	{
-		int rc, fd;
+		int rc, fd, n=0;
 
 		fd = open( URANDOM_DEVICE, O_RDONLY );
 
 		if( fd < 0 ) return -1;
 
-		rc = read( fd, buf, nbytes );
+		do {
+			rc = read( fd, buf, nbytes );
+			if( rc <= 0 ) break;
+
+			buf+=rc;
+			nbytes-=rc;
+
+			if( ++n >= URANDOM_NREADS ) break;
+		} while( nbytes > 0 );
+
 		close(fd);
-
-		/* should return nbytes */
-		if( rc != nbytes ) return -1;
-
-		return 0;
+		return nbytes > 0 ? -1 : 0;
 	}
 #elif PROV_RSA_FULL
 	{
@@ -116,7 +137,7 @@ int lutil_entropy( unsigned char *buf, ber_len_t nbytes )
 
 		for( n = 0; n < nbytes; n += 16 ) {
 			struct lutil_MD5Context ctx;
-			char digest[16];
+			unsigned char digest[16];
 
 			/* poor resolution */
 #ifdef HAVE_GETTIMEOFDAY
