@@ -1,8 +1,19 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/passwd.c,v 1.74.2.1 2003/09/18 15:43:33 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/passwd.c,v 1.74.2.3 2003/12/01 00:58:29 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/passwd.c,v 1.74.2.3 2003/12/01 00:58:29 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 /*
  * int lutil_passwd(
  *	const struct berval *passwd,
@@ -485,7 +496,7 @@ static int chk_ssha1(
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 
-	if (rc <= sizeof(SHA1digest)) {
+	if (rc < 0 || (unsigned)rc <= sizeof(SHA1digest)) {
 		ber_memfree(orig_pass);
 		return -1;
 	}
@@ -566,7 +577,7 @@ static int chk_smd5(
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 
-	if (rc <= sizeof(MD5digest)) {
+	if (rc < 0 || (unsigned)rc <= sizeof(MD5digest)) {
 		ber_memfree(orig_pass);
 		return -1;
 	}
@@ -762,7 +773,7 @@ static int chk_ns_mta_md5(
 	char buffer[LUTIL_MD5_BYTES*2];
 	int i;
 
-	if( passwd.bv_len != LUTIL_MD5_BYTES*2 ) {
+	if( passwd->bv_len != LUTIL_MD5_BYTES*2 ) {
 		return 1;
 	}
 
