@@ -1,8 +1,27 @@
 /* aclparse.c - routines to parse and check acl's */
-/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.110.2.2 2003/09/22 04:09:39 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.110.2.4 2003/12/01 00:58:31 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
  */
 
 #include "portable.h"
@@ -1565,7 +1584,7 @@ print_access( Access *b )
 			fprintf( stderr, " %s", b->a_dn_pat.bv_val );
 
 		} else {
-			fprintf( stderr, " dn.%s=%s",
+			fprintf( stderr, " dn.%s=\"%s\"",
 				style_strings[b->a_dn_style], b->a_dn_pat.bv_val );
 		}
 	}
@@ -1575,25 +1594,19 @@ print_access( Access *b )
 	}
 
 	if ( b->a_group_pat.bv_len ) {
-		fprintf( stderr, " group=%s", b->a_group_pat.bv_val );
-
-		if ( b->a_group_oc ) {
-			fprintf( stderr, " objectClass: %s",
-				b->a_group_oc->soc_oclass.oc_oid );
-
-			if ( b->a_group_at ) {
-				fprintf( stderr, " attributeType: %s",
-					b->a_group_at->ad_cname.bv_val );
-			}
-		}
+		fprintf( stderr, " group/%s/%s.%s=\"%s\"",
+			b->a_group_oc ? b->a_group_oc->soc_cname.bv_val : "groupOfNames",
+			b->a_group_at ? b->a_group_at->ad_cname.bv_val : "member",
+			style_strings[b->a_group_style],
+			b->a_group_pat.bv_val );
     }
 
 	if ( b->a_peername_pat.bv_len != 0 ) {
-		fprintf( stderr, " peername=%s", b->a_peername_pat.bv_val );
+		fprintf( stderr, " peername=\"%s\"", b->a_peername_pat.bv_val );
 	}
 
 	if ( b->a_sockname_pat.bv_len != 0 ) {
-		fprintf( stderr, " sockname=%s", b->a_sockname_pat.bv_val );
+		fprintf( stderr, " sockname=\"%s\"", b->a_sockname_pat.bv_val );
 	}
 
 	if ( b->a_domain_pat.bv_len != 0 ) {
@@ -1601,7 +1614,7 @@ print_access( Access *b )
 	}
 
 	if ( b->a_sockurl_pat.bv_len != 0 ) {
-		fprintf( stderr, " sockurl=%s", b->a_sockurl_pat.bv_val );
+		fprintf( stderr, " sockurl=\"%s\"", b->a_sockurl_pat.bv_val );
 	}
 
 #ifdef SLAPD_ACI_ENABLED
@@ -1657,7 +1670,7 @@ print_acl( Backend *be, AccessControl *a )
 
 	if ( a->acl_dn_pat.bv_len != 0 ) {
 		to++;
-		fprintf( stderr, " dn.%s=%s\n",
+		fprintf( stderr, " dn.%s=\"%s\"\n",
 			style_strings[a->acl_dn_style], a->acl_dn_pat.bv_val );
 	}
 
@@ -1687,7 +1700,7 @@ print_acl( Backend *be, AccessControl *a )
 
 	if ( a->acl_attrval.bv_len != 0 ) {
 		to++;
-		fprintf( stderr, " val.%s=%s\n",
+		fprintf( stderr, " val.%s=\"%s\"\n",
 			style_strings[a->acl_attrval_style], a->acl_attrval.bv_val );
 
 	}
