@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.162.2.2 2003/09/22 04:09:39 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.162.2.4 2003/12/01 00:58:31 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1995 Regents of the University of Michigan.
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -231,7 +239,7 @@ do_add( Operation *op, SlapReply *rs )
 	}
 
 #ifdef LDAP_SLAPI
-	initAddPlugin( op, &dn, e, manageDSAit );
+	if ( op->o_pb ) initAddPlugin( op, &dn, e, manageDSAit );
 #endif /* LDAP_SLAPI */
 
 	/*
@@ -244,10 +252,10 @@ do_add( Operation *op, SlapReply *rs )
 		/* do the update here */
 		int repl_user = be_isupdate(op->o_bd, &op->o_ndn );
 #ifndef SLAPD_MULTIMASTER
-		if ( !op->o_bd->syncinfo &&
-						( !op->o_bd->be_update_ndn.bv_len || repl_user ))
+		if ( LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo ) &&
+			( !op->o_bd->be_update_ndn.bv_len || repl_user ))
 #else
-		if ( !op->o_bd->syncinfo )
+		if ( LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo ))
 #endif
 		{
 			int update = op->o_bd->be_update_ndn.bv_len;
@@ -290,10 +298,12 @@ do_add( Operation *op, SlapReply *rs )
 			 * Call the preoperation plugin here, because the entry
 			 * will actually contain something.
 			 */
-			rs->sr_err = doPreAddPluginFNs( op );
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				/* plugin will have sent result */
-				goto done;
+			if ( op->o_pb ) {
+				rs->sr_err = doPreAddPluginFNs( op );
+				if ( rs->sr_err != LDAP_SUCCESS ) {
+					/* plugin will have sent result */
+					goto done;
+				}
 			}
 #endif /* LDAP_SLAPI */
 
@@ -317,18 +327,25 @@ do_add( Operation *op, SlapReply *rs )
 			 * SLAPI_ADD_ENTRY will be empty, but this may be acceptable
 			 * on replicas (for now, it involves the minimum code intrusion).
 			 */
-			rs->sr_err = doPreAddPluginFNs( op );
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				/* plugin will have sent result */
-				goto done;
+			if ( op->o_pb ) {
+				rs->sr_err = doPreAddPluginFNs( op );
+				if ( rs->sr_err != LDAP_SUCCESS ) {
+					/* plugin will have sent result */
+					goto done;
+				}
 			}
 #endif /* LDAP_SLAPI */
 
-			if ( op->o_bd->syncinfo ) {
-				defref = op->o_bd->syncinfo->provideruri_bv;
+			if ( !LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo )) {
+				syncinfo_t *si;
+				LDAP_STAILQ_FOREACH( si, &op->o_bd->be_syncinfo, si_next ) {
+					struct berval tmpbv;
+					ber_dupbv( &tmpbv, &si->si_provideruri_bv[0] );
+					ber_bvarray_add( &defref, &tmpbv );
+				}
 			} else {
 				defref = op->o_bd->be_update_refs
-							? op->o_bd->be_update_refs : default_referral;
+					? op->o_bd->be_update_refs : default_referral;
 			}
 
 			if ( defref != NULL ) {
@@ -349,10 +366,12 @@ do_add( Operation *op, SlapReply *rs )
 		}
 	} else {
 #ifdef LDAP_SLAPI
-	    rs->sr_err = doPreAddPluginFNs( op );
-	    if ( rs->sr_err != LDAP_SUCCESS ) {
-			/* plugin will have sent result */
-			goto done;
+		if ( op->o_pb ) {
+			rs->sr_err = doPreAddPluginFNs( op );
+			if ( rs->sr_err != LDAP_SUCCESS ) {
+				/* plugin will have sent result */
+				goto done;
+			}
 		}
 #endif
 #ifdef NEW_LOGGING
@@ -366,7 +385,7 @@ do_add( Operation *op, SlapReply *rs )
 	}
 
 #ifdef LDAP_SLAPI
-	doPostAddPluginFNs( op );
+	if ( op->o_pb ) doPostAddPluginFNs( op );
 #endif /* LDAP_SLAPI */
 
 done:
