@@ -1,8 +1,22 @@
 /* init.c - initialize ldap backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/init.c,v 1.43.2.1 2003/09/18 15:43:37 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/init.c,v 1.43.2.3 2003/12/01 00:58:40 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by the Howard Chu for inclusion
+ * in OpenLDAP Software and subsequently enhanced by Pierangelo
+ * Masarati.
  */
 /* This is an altered version */
 /*
@@ -198,7 +212,7 @@ ldap_back_db_destroy(
 		}
 #ifdef ENABLE_REWRITE
 		if (li->rwmap.rwm_rw) {
-			rewrite_info_delete( li->rwmap.rwm_rw );
+			rewrite_info_delete( &li->rwmap.rwm_rw );
 		}
 #else /* !ENABLE_REWRITE */
 		if (li->rwmap.rwm_suffix_massage) {
