@@ -1,7 +1,26 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.230.2.3 2003/10/06 14:57:29 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.230.2.5 2003/12/01 00:58:33 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
  */
 
 #include "portable.h"
@@ -39,16 +58,18 @@ static unsigned long conn_nextid = 0;
 #define SLAP_C_ACTIVE			0x02	/* one or more threads */
 #define SLAP_C_BINDING			0x03	/* binding */
 #define SLAP_C_CLOSING			0x04	/* closing */
+#define SLAP_C_CLIENT			0x05	/* outbound client conn */
 
 const char *
 connection_state2str( int state )
 {
 	switch( state ) {
-	case SLAP_C_INVALID:	return "!";		
-	case SLAP_C_INACTIVE:	return "|";		
-	case SLAP_C_ACTIVE:		return "";			
+	case SLAP_C_INVALID:	return "!";
+	case SLAP_C_INACTIVE:	return "|";
+	case SLAP_C_ACTIVE:		return "";
 	case SLAP_C_BINDING:	return "B";
-	case SLAP_C_CLOSING:	return "C";			
+	case SLAP_C_CLOSING:	return "C";
+	case SLAP_C_CLIENT:		return "L";
 	}
 
 	return "?";
@@ -76,7 +97,7 @@ int connections_init(void)
 	if( connections != NULL) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO,
-			   "connections_init:  already initialized.\n", 0, 0, 0 );
+			"connections_init: already initialized.\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "connections_init: already initialized.\n",
 			0, 0, 0 );
@@ -92,8 +113,8 @@ int connections_init(void)
 	if( connections == NULL ) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR,
-			   "connections_init: allocation (%d * %ld) of connection "
-			   "array failed\n", dtblsize, (long) sizeof(Connection), 0 );
+			"connections_init: allocation (%d * %ld) of connection "
+			"array failed\n", dtblsize, (long) sizeof(Connection), 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connections_init: allocation (%d*%ld) of connection array failed\n",
@@ -102,8 +123,8 @@ int connections_init(void)
 		return -1;
 	}
 
-    assert( connections[0].c_struct_state == SLAP_C_UNINITIALIZED );
-    assert( connections[dtblsize-1].c_struct_state == SLAP_C_UNINITIALIZED );
+	assert( connections[0].c_struct_state == SLAP_C_UNINITIALIZED );
+	assert( connections[dtblsize-1].c_struct_state == SLAP_C_UNINITIALIZED );
 
 	/*
 	 * per entry initialization of the Connection array initialization
@@ -125,7 +146,7 @@ int connections_destroy(void)
 	if( connections == NULL) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO,
-			   "connections_destroy: nothing to destroy.\n", 0, 0, 0 );
+			"connections_destroy: nothing to destroy.\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "connections_destroy: nothing to destroy.\n",
 			0, 0, 0 );
@@ -140,7 +161,9 @@ int connections_destroy(void)
 			ldap_pvt_thread_mutex_destroy( &connections[i].c_write_mutex );
 			ldap_pvt_thread_cond_destroy( &connections[i].c_write_cv );
 #ifdef LDAP_SLAPI
-			slapi_x_free_object_extensions( SLAPI_X_EXT_CONNECTION, &connections[i] );
+			if ( slapi_plugins_used ) {
+				slapi_x_free_object_extensions( SLAPI_X_EXT_CONNECTION, &connections[i] );
+			}
 #endif
 		}
 	}
@@ -165,6 +188,12 @@ int connections_shutdown(void)
 		if( connections[i].c_struct_state != SLAP_C_USED ) {
 			continue;
 		}
+		/* give persistent clients a chance to cleanup */
+		if( connections[i].c_conn_state == SLAP_C_CLIENT ) {
+			ldap_pvt_thread_pool_submit( &connection_pool,
+			connections[i].c_clientfunc, connections[i].c_clientarg );
+			continue;
+		}
 
 		ldap_pvt_thread_mutex_lock( &connections[i].c_mutex );
 
@@ -193,8 +222,10 @@ int connections_timeout_idle(time_t now)
 		c != NULL;
 		c = connection_next( c, &connindex ) )
 	{
-		/* Don't timeout a slow-running request */
-		if( c->c_n_ops_executing ) continue;
+		/* Don't timeout a slow-running request or a persistent
+		 * outbound connection */
+		if( c->c_n_ops_executing ||
+			c->c_conn_state == SLAP_C_CLIENT ) continue;
 
 		if( difftime( c->c_activitytime+global_idletimeout, now) < 0 ) {
 			/* close it */
@@ -280,7 +311,7 @@ static Connection* connection_get( ber_socket_t s )
 
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, ARGS, 
-				"connection_get:  connection %d not used\n", s, 0, 0 );
+				"connection_get: connection %d not used\n", s, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_get(%d): connection not used\n",
@@ -290,6 +321,7 @@ static Connection* connection_get( ber_socket_t s )
 			ldap_pvt_thread_mutex_unlock( &c->c_mutex );
 			return NULL;
 		}
+		if( c->c_conn_state == SLAP_C_CLIENT ) sd = 0;
 
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, RESULTS, 
@@ -328,7 +360,7 @@ long connection_init(
 	Listener *listener,
 	const char* dnsname,
 	const char* peername,
-	int tls_udp_option,
+	int flags,
 	slap_ssf_t ssf,
 	const char *authid )
 {
@@ -342,17 +374,16 @@ long connection_init(
 	assert( peername != NULL );
 
 #ifndef HAVE_TLS
-	assert( tls_udp_option != 1 );
+	assert( flags != CONN_IS_TLS );
 #endif
 
 	if( s == AC_SOCKET_INVALID ) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO, 
-			   "connection_init: init of socket %ld invalid.\n", (long)s, 0, 0 );
+			"connection_init: init of socket %ld invalid.\n", (long)s, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
-		       "connection_init(%ld): invalid.\n",
-		       (long) s, 0, 0 );
+			"connection_init: init of socket %ld invalid.\n", (long)s, 0, 0 );
 #endif
 		return -1;
 	}
@@ -369,48 +400,49 @@ long connection_init(
 
 #else
 	{
-		ber_socket_t	i;
-
+		ber_socket_t i;
 		c = NULL;
 
-	for( i=0; i < dtblsize; i++) {
-		ber_socket_t	sd;
+		for( i=0; i < dtblsize; i++) {
+			ber_socket_t	sd;
 
-	    if( connections[i].c_struct_state == SLAP_C_UNINITIALIZED ) {
-		assert( connections[i].c_sb == 0 );
-		c = &connections[i];
-		break;
-	    }
+			if( connections[i].c_struct_state == SLAP_C_UNINITIALIZED ) {
+				assert( connections[i].c_sb == 0 );
+				c = &connections[i];
+				break;
+			}
 
 			sd = AC_SOCKET_INVALID;
-			if (connections[i].c_sb != NULL)
-			ber_sockbuf_ctrl( connections[i].c_sb, LBER_SB_OPT_GET_FD, &sd );
-	    
-	    if( connections[i].c_struct_state == SLAP_C_UNUSED ) {
-		assert( sd == AC_SOCKET_INVALID );
-		c = &connections[i];
-		break;
-	    }
+			if (connections[i].c_sb != NULL) {
+				ber_sockbuf_ctrl( connections[i].c_sb,
+					LBER_SB_OPT_GET_FD, &sd );
+			}
 
-	    assert( connections[i].c_struct_state == SLAP_C_USED );
-	    assert( connections[i].c_conn_state != SLAP_C_INVALID );
-	    assert( sd != AC_SOCKET_INVALID );
-	}
+			if( connections[i].c_struct_state == SLAP_C_UNUSED ) {
+				assert( sd == AC_SOCKET_INVALID );
+				c = &connections[i];
+				break;
+			}
 
-	if( c == NULL ) {
+			assert( connections[i].c_struct_state == SLAP_C_USED );
+			assert( connections[i].c_conn_state != SLAP_C_INVALID );
+			assert( sd != AC_SOCKET_INVALID );
+		}
+
+		if( c == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG( CONNECTION, INFO, 
-			   "connection_init: skt %d connection table full "
-			   "(%d/%d)\n", s, i, dtblsize );
+			LDAP_LOG( CONNECTION, INFO, 
+				"connection_init(%d): connection table full "
+				"(%d/%d)\n", s, i, dtblsize );
 #else
-		Debug( LDAP_DEBUG_ANY,
+			Debug( LDAP_DEBUG_ANY,
 				"connection_init(%d): connection table full "
 				"(%d/%d)\n", s, i, dtblsize);
 #endif
-	    ldap_pvt_thread_mutex_unlock( &connections_mutex );
-	    return -1;
+			ldap_pvt_thread_mutex_unlock( &connections_mutex );
+			return -1;
+		}
 	}
-    	}
 #endif
 
 	assert( c != NULL );
@@ -430,7 +462,6 @@ long connection_init(
 		c->c_dn.bv_len = 0;
 		c->c_ndn.bv_val = NULL;
 		c->c_ndn.bv_len = 0;
-		c->c_groups = NULL;
 
 		c->c_listener = NULL;
 		c->c_peer_domain.bv_val = NULL;
@@ -464,24 +495,25 @@ long connection_init(
 		ldap_pvt_thread_cond_init( &c->c_write_cv );
 
 #ifdef LDAP_SLAPI
-		slapi_x_create_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+		if ( slapi_plugins_used ) {
+			slapi_x_create_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+		}
 #endif
 
 		c->c_struct_state = SLAP_C_UNUSED;
 	}
 
-    ldap_pvt_thread_mutex_lock( &c->c_mutex );
+	ldap_pvt_thread_mutex_lock( &c->c_mutex );
 
-    assert( c->c_struct_state == SLAP_C_UNUSED );
-    assert( c->c_authmech.bv_val == NULL );
-    assert( c->c_dn.bv_val == NULL );
-    assert( c->c_ndn.bv_val == NULL );
-    assert( c->c_groups == NULL );
-    assert( c->c_listener == NULL );
-    assert( c->c_peer_domain.bv_val == NULL );
-    assert( c->c_peer_name.bv_val == NULL );
-    assert( LDAP_STAILQ_EMPTY(&c->c_ops) );
-    assert( LDAP_STAILQ_EMPTY(&c->c_pending_ops) );
+	assert( c->c_struct_state == SLAP_C_UNUSED );
+	assert( c->c_authmech.bv_val == NULL );
+	assert( c->c_dn.bv_val == NULL );
+	assert( c->c_ndn.bv_val == NULL );
+	assert( c->c_listener == NULL );
+	assert( c->c_peer_domain.bv_val == NULL );
+	assert( c->c_peer_name.bv_val == NULL );
+	assert( LDAP_STAILQ_EMPTY(&c->c_ops) );
+	assert( LDAP_STAILQ_EMPTY(&c->c_pending_ops) );
 	assert( c->c_sasl_bind_mech.bv_val == NULL );
 	assert( c->c_sasl_done == 0 );
 	assert( c->c_sasl_authctx == NULL );
@@ -489,15 +521,26 @@ long connection_init(
 	assert( c->c_sasl_extra == NULL );
 	assert( c->c_sasl_bindop == NULL );
 	assert( c->c_currentber == NULL );
+	assert( c->c_writewaiter == 0);
 
 	c->c_listener = listener;
+
+	if ( flags == CONN_IS_CLIENT ) {
+		c->c_conn_state = SLAP_C_CLIENT;
+		c->c_struct_state = SLAP_C_USED;
+		ldap_pvt_thread_mutex_unlock( &c->c_mutex );
+		ldap_pvt_thread_mutex_unlock( &connections_mutex );
+
+		return 0;
+	}
+
 	ber_str2bv( dnsname, 0, 1, &c->c_peer_domain );
 	ber_str2bv( peername, 0, 1, &c->c_peer_name );
 
-    c->c_n_ops_received = 0;
-    c->c_n_ops_executing = 0;
-    c->c_n_ops_pending = 0;
-    c->c_n_ops_completed = 0;
+	c->c_n_ops_received = 0;
+	c->c_n_ops_executing = 0;
+	c->c_n_ops_pending = 0;
+	c->c_n_ops_completed = 0;
 
 	c->c_n_get = 0;
 	c->c_n_read = 0;
@@ -516,7 +559,7 @@ long connection_init(
 
 #ifdef LDAP_CONNECTIONLESS
 	c->c_is_udp = 0;
-	if( tls_udp_option == 2 ) {
+	if( flags == CONN_IS_UDP ) {
 		c->c_is_udp = 1;
 #ifdef LDAP_DEBUG
 		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_debug,
@@ -547,8 +590,8 @@ long connection_init(
 	{
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO, 
-			   "connection_init: conn %lu  set nonblocking failed\n",
-			   c->c_connid, 0, 0 );
+			"connection_init: conn %lu set nonblocking failed\n",
+			c->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connection_init(%d, %s): set nonblocking failed\n",
@@ -556,33 +599,33 @@ long connection_init(
 #endif
 	}
 
-    id = c->c_connid = conn_nextid++;
+	id = c->c_connid = conn_nextid++;
 
-    c->c_conn_state = SLAP_C_INACTIVE;
-    c->c_struct_state = SLAP_C_USED;
+	c->c_conn_state = SLAP_C_INACTIVE;
+	c->c_struct_state = SLAP_C_USED;
 
 	c->c_ssf = c->c_transport_ssf = ssf;
 	c->c_tls_ssf = 0;
 
 #ifdef HAVE_TLS
-    if ( tls_udp_option == 1 ) {
-	    c->c_is_tls = 1;
-	    c->c_needs_tls_accept = 1;
-    } else {
-	    c->c_is_tls = 0;
-	    c->c_needs_tls_accept = 0;
-    }
+	if ( flags == CONN_IS_TLS ) {
+		c->c_is_tls = 1;
+		c->c_needs_tls_accept = 1;
+	} else {
+		c->c_is_tls = 0;
+		c->c_needs_tls_accept = 0;
+	}
 #endif
 
 	slap_sasl_open( c, 0 );
 	slap_sasl_external( c, ssf, authid );
 
-    ldap_pvt_thread_mutex_unlock( &c->c_mutex );
-    ldap_pvt_thread_mutex_unlock( &connections_mutex );
+	ldap_pvt_thread_mutex_unlock( &c->c_mutex );
+	ldap_pvt_thread_mutex_unlock( &connections_mutex );
 
-    backend_connection_init(c);
+	backend_connection_init(c);
 
-    return id;
+	return id;
 }
 
 void connection2anonymous( Connection *c )
@@ -613,39 +656,31 @@ void connection2anonymous( Connection *c )
 	c->c_ndn.bv_len = 0;
 
 	c->c_authz_backend = NULL;
-	
-	{
-		GroupAssertion *g, *n;
-		for (g = c->c_groups; g; g=n) {
-			n = g->ga_next;
-			free(g);
-		}
-		c->c_groups = NULL;
-	}
 }
 
 static void
 connection_destroy( Connection *c )
 {
 	/* note: connections_mutex should be locked by caller */
-    ber_socket_t	sd;
-    unsigned long	connid;
+	ber_socket_t	sd;
+	unsigned long	connid;
 
-    assert( connections != NULL );
-    assert( c != NULL );
-    assert( c->c_struct_state != SLAP_C_UNUSED );
-    assert( c->c_conn_state != SLAP_C_INVALID );
-    assert( LDAP_STAILQ_EMPTY(&c->c_ops) );
+	assert( connections != NULL );
+	assert( c != NULL );
+	assert( c->c_struct_state != SLAP_C_UNUSED );
+	assert( c->c_conn_state != SLAP_C_INVALID );
+	assert( LDAP_STAILQ_EMPTY(&c->c_ops) );
+	assert( c->c_writewaiter == 0);
 
-    /* only for stats (print -1 as "%lu" may give unexpected results ;) */
-    connid = c->c_connid;
+	/* only for stats (print -1 as "%lu" may give unexpected results ;) */
+	connid = c->c_connid;
 
-    backend_connection_destroy(c);
+	backend_connection_destroy(c);
 
-    c->c_protocol = 0;
-    c->c_connid = -1;
+	c->c_protocol = 0;
+	c->c_connid = -1;
 
-    c->c_activitytime = c->c_starttime = 0;
+	c->c_activitytime = c->c_starttime = 0;
 
 	connection2anonymous( c );
 	c->c_listener = NULL;
@@ -677,10 +712,10 @@ connection_destroy( Connection *c )
 
 	ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_GET_FD, &sd );
 	if ( sd != AC_SOCKET_INVALID ) {
-		slapd_remove( sd, 0 );
+		slapd_remove( sd, 1, 0 );
 
 		Statslog( LDAP_DEBUG_STATS,
-		    "conn=%lu fd=%ld closed\n",
+			"conn=%lu fd=%ld closed\n",
 			connid, (long) sd, 0, 0, 0 );
 	}
 
@@ -693,12 +728,14 @@ connection_destroy( Connection *c )
 		ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_SET_MAX_INCOMING, &max );
 	}
 
-    c->c_conn_state = SLAP_C_INVALID;
-    c->c_struct_state = SLAP_C_UNUSED;
+	c->c_conn_state = SLAP_C_INVALID;
+	c->c_struct_state = SLAP_C_UNUSED;
 
 #ifdef LDAP_SLAPI
 	/* call destructors, then constructors; avoids unnecessary allocation */
-	slapi_x_clear_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+	if ( slapi_plugins_used ) {
+		slapi_x_clear_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+	}
 #endif
 }
 
@@ -750,8 +787,8 @@ void connection_closing( Connection *c )
 		ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_GET_FD, &sd );
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, DETAIL1, 
-			   "connection_closing: conn %lu readying socket %d for close.\n",
-			   c->c_connid, sd, 0 );
+			"connection_closing: conn %lu readying socket %d for close.\n",
+			c->c_connid, sd, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_closing: readying conn=%lu sd=%d for close\n",
@@ -787,8 +824,8 @@ static void connection_close( Connection *c )
 	if( !LDAP_STAILQ_EMPTY(&c->c_ops) ) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, DETAIL1, 
-			   "connection_close: conn %lu  deferring sd %d\n",
-			   c->c_connid, sd, 0 );
+			"connection_close: conn %lu deferring sd %d\n",
+				c->c_connid, sd, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_close: deferring conn=%lu sd=%d\n",
@@ -799,7 +836,7 @@ static void connection_close( Connection *c )
 
 #ifdef NEW_LOGGING
 	LDAP_LOG( CONNECTION, RESULTS, 
-		   "connection_close: conn %lu  sd %d\n", c->c_connid, sd, 0 );
+		"connection_close: conn %lu sd %d\n", c->c_connid, sd, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "connection_close: conn=%lu sd=%d\n",
 		c->c_connid, sd, 0 );
@@ -912,6 +949,7 @@ connection_operation( void *ctx, void *arg_v )
 #endif /* SLAPD_MONITOR */
 	Connection *conn = op->o_conn;
 	void *memctx = NULL;
+	void *memctx_null = NULL;
 	ber_len_t memsiz;
 
 	ldap_pvt_thread_mutex_lock( &num_ops_mutex );
@@ -1010,11 +1048,11 @@ connection_operation( void *ctx, void *arg_v )
 	default:
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO, 
-			   "connection_operation: conn %lu  unknown LDAP request 0x%lx\n",
-			   conn->c_connid, tag, 0  );
+			"connection_operation: conn %lu unknown LDAP request 0x%lx\n",
+			conn->c_connid, tag, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "unknown LDAP request 0x%lx\n",
-		    tag, 0, 0 );
+			tag, 0, 0 );
 #endif
 		op->o_tag = LBER_ERROR;
 		rs.sr_err = LDAP_PROTOCOL_ERROR;
@@ -1082,26 +1120,25 @@ operations_error:
 
 	ldap_pvt_thread_mutex_lock( &conn->c_mutex );
 
-	LDAP_STAILQ_REMOVE( &conn->c_ops, op, slap_op, o_next);
-	LDAP_STAILQ_NEXT(op, o_next) = NULL;
+	ber_set_option( op->o_ber, LBER_OPT_BER_MEMCTX, &memctx_null );
 
-	if ( op->o_cancel == SLAP_CANCEL_ACK )
-		goto co_op_free;
-	if ( ( op->o_sync_mode & SLAP_SYNC_PERSIST ) ) {
+	if ( op->o_cancel != SLAP_CANCEL_ACK &&
+				( op->o_sync_mode & SLAP_SYNC_PERSIST ) ) {
 		sl_mem_detach( ctx, memctx );
-		goto no_co_op_free;
+	} else if (( op->o_sync_slog_size != -1 )) {
+		sl_mem_detach( ctx, memctx );
+		LDAP_STAILQ_REMOVE( &conn->c_ops, op, slap_op, o_next);
+		LDAP_STAILQ_NEXT(op, o_next) = NULL;
+		conn->c_n_ops_executing--;
+		conn->c_n_ops_completed++;
+	} else {
+		LDAP_STAILQ_REMOVE( &conn->c_ops, op, slap_op, o_next);
+		LDAP_STAILQ_NEXT(op, o_next) = NULL;
+		slap_op_free( op );
+		conn->c_n_ops_executing--;
+		conn->c_n_ops_completed++;
 	}
 
-co_op_free:
-
-	conn->c_n_ops_executing--;
-	conn->c_n_ops_completed++;
-	memctx = NULL;
-	ber_set_option( op->o_ber, LBER_OPT_BER_MEMCTX, &memctx );
-	slap_op_free( op );
-
-no_co_op_free:
-
 	switch( tag ) {
 	case LBER_ERROR:
 	case LDAP_REQ_UNBIND:
@@ -1125,6 +1162,52 @@ no_co_op_free:
 	return NULL;
 }
 
+int connection_client_setup(
+	ber_socket_t s,
+	Listener *l,
+	ldap_pvt_thread_start_t *func,
+	void *arg )
+{
+	Connection *c;
+
+	if ( connection_init( s, l, "", "", CONN_IS_CLIENT, 0, "" ) < 0 ) {
+		return -1;
+	}
+
+	c = connection_get( s );
+	c->c_clientfunc = func;
+	c->c_clientarg = arg;
+	connection_return( c );
+	slapd_add_internal( s, 0 );
+	slapd_set_read( s, 1 );
+	return 0;
+}
+
+void connection_client_enable(
+	ber_socket_t s
+)
+{
+	slapd_set_read( s, 1 );
+}
+
+void connection_client_stop(
+	ber_socket_t s
+)
+{
+	Connection *c;
+
+	/* get (locked) connection */
+	c = connection_get( s );
+	
+	assert( c->c_conn_state == SLAP_C_CLIENT );
+
+	c->c_listener = NULL;
+	c->c_conn_state = SLAP_C_INVALID;
+	c->c_struct_state = SLAP_C_UNUSED;
+	connection_return( c );
+	slapd_remove( s, 0, 1 );
+}
+
 int connection_read(ber_socket_t s)
 {
 	int rc = 0;
@@ -1146,7 +1229,7 @@ int connection_read(ber_socket_t s)
 			"connection_read(%ld): no connection!\n",
 			(long) s, 0, 0 );
 #endif
-		slapd_remove(s, 0);
+		slapd_remove(s, 1, 0);
 
 		ldap_pvt_thread_mutex_unlock( &connections_mutex );
 		return -1;
@@ -1169,10 +1252,19 @@ int connection_read(ber_socket_t s)
 		return 0;
 	}
 
+	if ( c->c_conn_state == SLAP_C_CLIENT ) {
+		slapd_clr_read( s, 0 );
+		ldap_pvt_thread_pool_submit( &connection_pool,
+			c->c_clientfunc, c->c_clientarg );
+		connection_return( c );
+		ldap_pvt_thread_mutex_unlock( &connections_mutex );
+		return 0;
+	}
+
 #ifdef NEW_LOGGING
 	LDAP_LOG( CONNECTION, DETAIL1, 
-		   "connection_read: conn %lu  checking for input.\n", 
-		   c->c_connid, 0, 0  );
+		"connection_read: conn %lu checking for input.\n", 
+			c->c_connid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"connection_read(%d): checking for input on id=%lu\n",
@@ -1190,8 +1282,8 @@ int connection_read(ber_socket_t s)
 
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, ERR, 
-				   "connection_read: conn %lu  TLS accept error, error %d\n",
-				   c->c_connid, rc, 0 );
+				"connection_read: conn %lu TLS accept error, error %d\n",
+				c->c_connid, rc, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_read(%d): TLS accept error "
@@ -1208,10 +1300,10 @@ int connection_read(ber_socket_t s)
 			FD_ZERO(&rfd);
 			FD_SET(s, &rfd);
 			for (rc=1; rc>0;) {
-			    tv.tv_sec = 1;
-			    tv.tv_usec = 0;
-			    rc = select(s+1, &rfd, NULL, NULL, &tv);
-			    if (rc == 1) {
+				tv.tv_sec = 1;
+				tv.tv_usec = 0;
+				rc = select(s+1, &rfd, NULL, NULL, &tv);
+				if (rc == 1) {
 					ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_DRAIN, NULL);
 				}
 			}
@@ -1271,7 +1363,7 @@ int connection_read(ber_socket_t s)
 
 		c->c_sasl_layers = 0;
 
-		rc = ldap_pvt_sasl_install( c->c_sb,  c->c_sasl_sockctx );
+		rc = ldap_pvt_sasl_install( c->c_sb, c->c_sasl_sockctx );
 
 		if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
@@ -1313,7 +1405,7 @@ int connection_read(ber_socket_t s)
 	if( rc < 0 ) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			"connection_read: conn %lu  input error %d, closing.\n",
+			"connection_read: conn %lu input error %d, closing.\n",
 			c->c_connid, rc, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
@@ -1343,7 +1435,7 @@ int connection_read(ber_socket_t s)
 
 static int
 connection_input(
-    Connection *conn
+	Connection *conn
 )
 {
 	Operation *op;
@@ -1362,7 +1454,7 @@ connection_input(
 	{
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			"connection_input: conn %lu  ber_alloc failed.\n", 
+			"connection_input: conn %lu ber_alloc failed.\n", 
 			conn->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ber_alloc failed\n", 0, 0, 0 );
@@ -1396,7 +1488,7 @@ connection_input(
 
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			"connection_input: conn %lu  ber_get_next failed, errno %d (%s).\n",
+			"connection_input: conn %lu ber_get_next failed, errno %d (%s).\n",
 			conn->c_connid, err, sock_errstr(err) );
 #else
 		Debug( LDAP_DEBUG_TRACE,
@@ -1420,11 +1512,11 @@ connection_input(
 		/* log, close and send error */
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			"connection_input: conn %lu  ber_get_int returns 0x%lx.\n",
+			"connection_input: conn %lu ber_get_int returns 0x%lx.\n",
 			conn->c_connid, tag, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "ber_get_int returns 0x%lx\n", tag, 0,
-		    0 );
+		Debug( LDAP_DEBUG_ANY, "ber_get_int returns 0x%lx\n",
+			tag, 0, 0 );
 #endif
 		ber_free( ber, 1 );
 		return -1;
@@ -1434,11 +1526,11 @@ connection_input(
 		/* log, close and send error */
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			   "connection_input: conn %lu  ber_peek_tag returns 0x%lx.\n",
-			   conn->c_connid, tag, 0 );
+			"connection_input: conn %lu ber_peek_tag returns 0x%lx.\n",
+			conn->c_connid, tag, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "ber_peek_tag returns 0x%lx\n", tag, 0,
-		    0 );
+		Debug( LDAP_DEBUG_ANY, "ber_peek_tag returns 0x%lx\n",
+			tag, 0, 0 );
 #endif
 		ber_free( ber, 1 );
 
@@ -1453,15 +1545,14 @@ connection_input(
 		}
 		if( tag != LDAP_REQ_ABANDON && tag != LDAP_REQ_SEARCH ) {
 #ifdef NEW_LOGGING
-		    LDAP_LOG( CONNECTION, ERR, 
-			       "connection_input: conn %lu  invalid req for UDP 0x%lx.\n",
-			       conn->c_connid, tag, 0 );
+			LDAP_LOG( CONNECTION, ERR, 
+				"connection_input: conn %lu invalid req for UDP 0x%lx.\n",
+				conn->c_connid, tag, 0 );
 #else
-		    Debug( LDAP_DEBUG_ANY, "invalid req for UDP 0x%lx\n", tag, 0,
-			0 );
+			Debug( LDAP_DEBUG_ANY, "invalid req for UDP 0x%lx\n", tag, 0, 0 );
 #endif
-		    ber_free( ber, 1 );
-		    return 0;
+			ber_free( ber, 1 );
+			return 0;
 		}
 	}
 #endif
@@ -1487,8 +1578,8 @@ connection_input(
 #ifdef LDAP_CONNECTIONLESS
 	if (conn->c_is_udp) {
 		if ( cdn ) {
-		    ber_str2bv( cdn, 0, 1, &op->o_dn );
-		    op->o_protocol = LDAP_VERSION2;
+			ber_str2bv( cdn, 0, 1, &op->o_dn );
+			op->o_protocol = LDAP_VERSION2;
 		}
 		op->o_res_ber = ber_alloc_t( LBER_USE_DER );
 		if (op->o_res_ber == NULL) return 1;
@@ -1499,7 +1590,7 @@ connection_input(
 		if (rc != sizeof(struct sockaddr)) {
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, INFO, 
-				"connection_input: conn %lu  ber_write failed\n",
+				"connection_input: conn %lu ber_write failed\n",
 				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "ber_write failed\n", 0, 0, 0 );
@@ -1512,7 +1603,7 @@ connection_input(
 			if (rc == -1) {
 #ifdef NEW_LOGGING
 				LDAP_LOG( CONNECTION, INFO, 
-					"connection_input: conn %lu  put outer sequence failed\n",
+					"connection_input: conn %lu put outer sequence failed\n",
 					conn->c_connid, 0, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY, "ber_write failed\n", 0, 0, 0 );
@@ -1539,14 +1630,18 @@ connection_input(
 		|| conn->c_n_ops_pending
 		|| conn->c_writewaiter))
 	{
-		int max = conn->c_dn.bv_len ? slap_conn_max_pending_auth
-			 : slap_conn_max_pending;
+		int max = conn->c_dn.bv_len
+			? slap_conn_max_pending_auth
+			: slap_conn_max_pending;
+
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, INFO, 
-			"connection_input: conn %lu  deferring operation\n",
+			"connection_input: conn %lu deferring operation\n",
 			conn->c_connid, 0, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "deferring operation\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ANY,
+			"connection_input: conn=%lu deferring operation\n",
+			conn->c_connid, 0, 0 );
 #endif
 		conn->c_n_ops_pending++;
 		LDAP_STAILQ_INSERT_TAIL( &conn->c_pending_ops, op, o_next );
@@ -1587,7 +1682,7 @@ connection_resched( Connection *conn )
 		if( rc ) {
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, DETAIL1, 
-				"connection_resched: conn %lu  reaquiring locks.\n",
+				"connection_resched: conn %lu reaquiring locks.\n",
 				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
@@ -1607,7 +1702,7 @@ connection_resched( Connection *conn )
 		if( conn->c_conn_state != SLAP_C_CLOSING ) {
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, INFO, 
-				"connection_resched: conn %lu  closed by other thread.\n",
+				"connection_resched: conn %lu closed by other thread.\n",
 				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "connection_resched: "
@@ -1617,7 +1712,7 @@ connection_resched( Connection *conn )
 		} else {
 #ifdef NEW_LOGGING
 			LDAP_LOG( CONNECTION, DETAIL1, 
-				"connection_resched: conn %lu  attempting closing.\n",
+				"connection_resched: conn %lu attempting closing.\n",
 				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "connection_resched: "
@@ -1667,19 +1762,20 @@ static int connection_op_activate( Operation *op )
 	}
 
 	if (!op->o_dn.bv_len) {
-	    op->o_authz = op->o_conn->c_authz;
-	    ber_dupbv( &op->o_dn, &op->o_conn->c_dn );
-	    ber_dupbv( &op->o_ndn, &op->o_conn->c_ndn );
+		op->o_authz = op->o_conn->c_authz;
+		ber_dupbv( &op->o_dn, &op->o_conn->c_dn );
+		ber_dupbv( &op->o_ndn, &op->o_conn->c_ndn );
 	}
 	op->o_authtype = op->o_conn->c_authtype;
 	ber_dupbv( &op->o_authmech, &op->o_conn->c_authmech );
 	
 	if (!op->o_protocol) {
-	    op->o_protocol = op->o_conn->c_protocol
-		? op->o_conn->c_protocol : LDAP_VERSION3;
+		op->o_protocol = op->o_conn->c_protocol
+			? op->o_conn->c_protocol : LDAP_VERSION3;
 	}
 	if (op->o_conn->c_conn_state == SLAP_C_INACTIVE
-		&& op->o_protocol > LDAP_VERSION2) {
+		&& op->o_protocol > LDAP_VERSION2)
+	{
 		op->o_conn->c_conn_state = SLAP_C_ACTIVE;
 	}
 
@@ -1697,7 +1793,8 @@ static int connection_op_activate( Operation *op )
 			op->o_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
-		"ldap_pvt_thread_pool_submit failed (%d)\n", status, 0, 0 );
+			"ldap_pvt_thread_pool_submit: failed (%d) for conn=%lu\n",
+			status, op->o_connid, 0 );
 #endif
 		/* should move op to pending list */
 	}
@@ -1720,13 +1817,13 @@ int connection_write(ber_socket_t s)
 	if( c == NULL ) {
 #ifdef NEW_LOGGING
 		LDAP_LOG( CONNECTION, ERR, 
-			"connection_write: sock %ld  no connection!\n", (long)s, 0, 0);
+			"connection_write: sock %ld no connection!\n", (long)s, 0, 0);
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connection_write(%ld): no connection!\n",
-			(long) s, 0, 0 );
+			(long)s, 0, 0 );
 #endif
-		slapd_remove(s, 0);
+		slapd_remove(s, 1, 0);
 		ldap_pvt_thread_mutex_unlock( &connections_mutex );
 		return -1;
 	}
@@ -1735,7 +1832,7 @@ int connection_write(ber_socket_t s)
 
 #ifdef NEW_LOGGING
 	LDAP_LOG( CONNECTION, DETAIL1, 
-		"connection_write conn %lu  waking output.\n", c->c_connid, 0, 0 );
+		"connection_write conn %lu waking output.\n", c->c_connid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"connection_write(%d): waking output for id=%lu\n",
@@ -1743,10 +1840,12 @@ int connection_write(ber_socket_t s)
 #endif
 	ldap_pvt_thread_cond_signal( &c->c_write_cv );
 
-	if ( ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_NEEDS_READ, NULL ) )
+	if ( ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_NEEDS_READ, NULL ) ) {
 		slapd_set_read( s, 1 );
-	if ( ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_NEEDS_WRITE, NULL ) )
+	}
+	if ( ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_NEEDS_WRITE, NULL ) ) {
 		slapd_set_write( s, 1 );
+	}
 	connection_return( c );
 	ldap_pvt_thread_mutex_unlock( &connections_mutex );
 	return 0;
