@@ -1,23 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/ctxcsn.c,v 1.9.2.2 2003/10/06 14:57:29 kurt Exp $ */
-/*
- * Context CSN Management Routines
- */
-/* Copyright (c) 2003 by International Business Machines, Inc.
+/* ctxcsn.c -- Context CSN Management Routines */
+/* $OpenLDAP: pkg/ldap/servers/slapd/ctxcsn.c,v 1.9.2.4 2003/12/01 00:58:33 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * International Business Machines, Inc. (hereinafter called IBM) grants
- * permission under its copyrights to use, copy, modify, and distribute this
- * Software with or without fee, provided that the above copyright notice and
- * all paragraphs of this notice appear in all copies, and that the name of IBM
- * not be used in connection with the marketing of any product incorporating
- * the Software or modifications thereof, without specific, written prior
- * permission.
+ * Copyright 2003 The OpenLDAP Foundation.
+ * Portions Copyright 2003 IBM Corporation.
+ * All rights reserved.
  *
- * THE SOFTWARE IS PROVIDED "AS IS", AND IBM DISCLAIMS ALL WARRANTIES,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
- * PARTICULAR PURPOSE.  IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL,
- * DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER ARISING
- * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE, EVEN
- * IF IBM IS APPRISED OF THE POSSIBILITY OF SUCH DAMAGES.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -26,7 +21,6 @@
 
 #include <ac/string.h>
 #include <ac/socket.h>
-#include <db.h>
 
 #include "ldap_pvt.h"
 #include "lutil.h"
@@ -48,8 +42,7 @@ slap_get_commit_csn( Operation *op, struct berval *csn )
 	ldap_pvt_thread_mutex_lock( &op->o_bd->be_pcl_mutex );
 
 	LDAP_TAILQ_FOREACH( csne, &op->o_bd->be_pending_csn_list, csn_link ) {
-		if ( csne->opid == op->o_opid && csne->connid == op->o_connid )
-			break;
+		if ( csne->opid == op->o_opid && csne->connid == op->o_connid ) break;
 	}
 
 	if ( csne ) {
@@ -57,10 +50,8 @@ slap_get_commit_csn( Operation *op, struct berval *csn )
 	}
 
 	LDAP_TAILQ_FOREACH( csne, &op->o_bd->be_pending_csn_list, csn_link ) {
-		if ( csne->state == SLAP_CSN_COMMIT )
-			committed_csne = csne;
-		if ( csne->state == SLAP_CSN_PENDING )
-			break;
+		if ( csne->state == SLAP_CSN_COMMIT ) committed_csne = csne;
+		if ( csne->state == SLAP_CSN_PENDING ) break;
 	}
 
 	ldap_pvt_thread_mutex_unlock( &op->o_bd->be_pcl_mutex );
@@ -78,8 +69,7 @@ slap_rewind_commit_csn( Operation *op )
 	ldap_pvt_thread_mutex_lock( &op->o_bd->be_pcl_mutex );
 
 	LDAP_TAILQ_FOREACH( csne, &op->o_bd->be_pending_csn_list, csn_link ) {
-		if ( csne->opid == op->o_opid && csne->connid == op->o_connid )
-			break;
+		if ( csne->opid == op->o_opid && csne->connid == op->o_connid ) break;
 	}
 
 	if ( csne ) {
@@ -103,8 +93,7 @@ slap_graduate_commit_csn( Operation *op )
 	ldap_pvt_thread_mutex_lock( &op->o_bd->be_pcl_mutex );
 
 	LDAP_TAILQ_FOREACH( csne, &op->o_bd->be_pending_csn_list, csn_link ) {
-		if ( csne->opid == op->o_opid && csne->connid == op->o_connid )
-			break;
+		if ( csne->opid == op->o_opid && csne->connid == op->o_connid ) break;
 	}
 
 	if ( csne ) {
@@ -137,13 +126,14 @@ slap_create_context_csn_entry(
 
 	struct berval bv;
 
-	e = ( Entry * ) ch_calloc( 1, sizeof( Entry ));
+	e = (Entry *) ch_calloc( 1, sizeof( Entry ));
 
-	attr_merge( e, slap_schema.si_ad_objectClass, ocbva, NULL );
-
-	attr_merge_one( e, slap_schema.si_ad_structuralObjectClass, &ocbva[1], NULL );
-
-	attr_merge_one( e, slap_schema.si_ad_cn, (struct berval *)&slap_ldapsync_bv, NULL );
+	attr_merge( e, slap_schema.si_ad_objectClass,
+		ocbva, NULL );
+	attr_merge_one( e, slap_schema.si_ad_structuralObjectClass,
+		&ocbva[1], NULL );
+	attr_merge_one( e, slap_schema.si_ad_cn,
+		(struct berval *)&slap_ldapsync_bv, NULL );
 
 	if ( context_csn ) {
 		attr_merge_one( e, slap_schema.si_ad_contextCSN,
@@ -154,7 +144,8 @@ slap_create_context_csn_entry(
 	bv.bv_len = sizeof("{}")-1;
 	attr_merge_one( e, slap_schema.si_ad_subtreeSpecification, &bv, NULL );
 
-	build_new_dn( &e->e_name, &be->be_nsuffix[0], (struct berval *)&slap_ldapsync_cn_bv, NULL );
+	build_new_dn( &e->e_name, &be->be_nsuffix[0],
+		(struct berval *)&slap_ldapsync_cn_bv, NULL );
 	ber_dupbv( &e->e_nname, &e->e_name );
 
 	return e;
@@ -183,25 +174,24 @@ slap_get_csn(
 	int manage_ctxcsn
 )
 {
-	struct	slap_csn_entry *pending;
+	struct slap_csn_entry *pending;
 
-	if ( csn == NULL )
-		return LDAP_OTHER;
-
-	if ( manage_ctxcsn ) {
-		pending = (struct slap_csn_entry *) ch_calloc( 1, sizeof( struct slap_csn_entry ));
-	}
+	if ( csn == NULL ) return LDAP_OTHER;
 
 	csn->bv_len = lutil_csnstr( csnbuf, len, 0, 0 );
 	csn->bv_val = csnbuf;
 
 	if ( manage_ctxcsn ) {
+		pending = (struct slap_csn_entry *) ch_calloc( 1,
+			sizeof( struct slap_csn_entry ));
 		ldap_pvt_thread_mutex_lock( &op->o_bd->be_pcl_mutex );
+		ber_dupbv( &op->o_sync_csn, csn );
 		pending->csn = ber_dupbv( NULL, csn );
 		pending->connid = op->o_connid;
 		pending->opid = op->o_opid;
 		pending->state = SLAP_CSN_PENDING;
-		LDAP_TAILQ_INSERT_TAIL( &op->o_bd->be_pending_csn_list, pending, csn_link );
+		LDAP_TAILQ_INSERT_TAIL( &op->o_bd->be_pending_csn_list,
+			pending, csn_link );
 		ldap_pvt_thread_mutex_unlock( &op->o_bd->be_pcl_mutex );
 	}
 
