@@ -1,7 +1,26 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.273.2.3 2003/10/13 01:57:02 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.273.2.5 2003/12/01 00:58:33 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
  */
 
 #include "portable.h"
@@ -195,8 +214,13 @@ void slapd_slp_dereg() {
 
 /*
  * Add a descriptor to daemon control
+ *
+ * If isactive, the descriptor is a live server session and is subject
+ * to idletimeout control. Otherwise, the descriptor is a passive
+ * listener or an outbound client session, and not subject to
+ * idletimeout.
  */
-static void slapd_add(ber_socket_t s) {
+static void slapd_add(ber_socket_t s, int isactive) {
 	ldap_pvt_thread_mutex_lock( &slap_daemon.sd_mutex );
 
 	assert( !FD_ISSET( s, &slap_daemon.sd_actives ));
@@ -209,7 +233,9 @@ static void slapd_add(ber_socket_t s) {
 	}
 #endif
 
-	slap_daemon.sd_nactives++;
+	if ( isactive ) {
+		slap_daemon.sd_nactives++;
+	}
 
 	FD_SET( s, &slap_daemon.sd_actives );
 	FD_SET( s, &slap_daemon.sd_readers );
@@ -231,10 +257,12 @@ static void slapd_add(ber_socket_t s) {
 /*
  * Remove the descriptor from daemon control
  */
-void slapd_remove(ber_socket_t s, int wake) {
+void slapd_remove(ber_socket_t s, int wasactive, int wake) {
 	ldap_pvt_thread_mutex_lock( &slap_daemon.sd_mutex );
 
-	slap_daemon.sd_nactives--;
+	if ( wasactive ) {
+		slap_daemon.sd_nactives--;
+	}
 
 #ifdef NEW_LOGGING
 	LDAP_LOG( CONNECTION, DETAIL1, 
@@ -1119,7 +1147,7 @@ close_listeners(
 	for ( l = 0; slap_listeners[l] != NULL; l++ ) {
 		if ( slap_listeners[l]->sl_sd != AC_SOCKET_INVALID ) {
 			if ( remove )
-				slapd_remove( slap_listeners[l]->sl_sd, 0 );
+				slapd_remove( slap_listeners[l]->sl_sd, 0, 0 );
 #ifdef LDAP_PF_LOCAL
 			if ( slap_listeners[l]->sl_sa.sa_addr.sa_family == AF_LOCAL ) {
 				unlink( slap_listeners[l]->sl_sa.sa_un_addr.sun_path );
@@ -1171,7 +1199,7 @@ slapd_daemon_task(
 		 * are unnecessary.
 		 */
 		if ( slap_listeners[l]->sl_is_udp ) {
-			slapd_add( slap_listeners[l]->sl_sd );
+			slapd_add( slap_listeners[l]->sl_sd, 1 );
 			continue;
 		}
 #endif
@@ -1228,7 +1256,7 @@ slapd_daemon_task(
 			return( (void*)-1 );
 		}
 
-		slapd_add( slap_listeners[l]->sl_sd );
+		slapd_add( slap_listeners[l]->sl_sd, 0 );
 	}
 
 #ifdef HAVE_NT_SERVICE_MANAGER
@@ -1332,11 +1360,6 @@ slapd_daemon_task(
 
 		ldap_pvt_thread_mutex_unlock( &slap_daemon.sd_mutex );
 
-		if ( !at ) {
-			at = ldap_pvt_thread_pool_backload(&connection_pool) -
-				 ldap_pvt_runqueue_persistent_backload( &syncrepl_rq );
-		}
-
 		if ( at 
 #if defined(HAVE_YIELDING_SELECT) || defined(NO_THREADS)
 			&&  ( tv.tv_sec || tv.tv_usec )
@@ -1350,15 +1373,15 @@ slapd_daemon_task(
 		rtask = ldap_pvt_runqueue_next_sched( &syncrepl_rq, &cat );
 		while ( cat && cat->tv_sec && cat->tv_sec <= now ) {
 			if ( ldap_pvt_runqueue_isrunning( &syncrepl_rq, rtask )) {
-				ldap_pvt_runqueue_resched( &syncrepl_rq, rtask );
+				ldap_pvt_runqueue_resched( &syncrepl_rq, rtask, 0 );
 			} else {
 				ldap_pvt_runqueue_runtask( &syncrepl_rq, rtask );
-				ldap_pvt_runqueue_resched( &syncrepl_rq, rtask );
+				ldap_pvt_runqueue_resched( &syncrepl_rq, rtask, 0 );
 				ldap_pvt_thread_mutex_unlock( &syncrepl_rq.rq_mutex );
 				ldap_pvt_thread_pool_submit( &connection_pool,
 											rtask->routine, (void *) rtask );
+				ldap_pvt_thread_mutex_lock( &syncrepl_rq.rq_mutex );
 			}
-			ldap_pvt_thread_mutex_lock( &syncrepl_rq.rq_mutex );
 			rtask = ldap_pvt_runqueue_next_sched( &syncrepl_rq, &cat );
 		}
 		ldap_pvt_thread_mutex_unlock( &syncrepl_rq.rq_mutex );
@@ -1503,7 +1526,7 @@ slapd_daemon_task(
 				    id = connection_init(
 					slap_listeners[l]->sl_sd,
 				    	slap_listeners[l], "", "",
-					2, ssf, authid );
+					CONN_IS_UDP, ssf, authid );
 				    slap_listeners[l]->sl_is_udp++;
 				}
 				continue;
@@ -1733,7 +1756,7 @@ slapd_daemon_task(
 				dnsname != NULL ? dnsname : SLAP_STRING_UNKNOWN,
 				peername,
 #ifdef HAVE_TLS
-				slap_listeners[l]->sl_is_tls,
+				slap_listeners[l]->sl_is_tls ? CONN_IS_TLS : 0,
 #else
 				0,
 #endif
@@ -1769,7 +1792,7 @@ slapd_daemon_task(
 				slap_listeners[l]->sl_name.bv_val,
 				0 );
 
-			slapd_add( s );
+			slapd_add( s, 1 );
 			continue;
 		}
 
@@ -2077,19 +2100,11 @@ slap_sig_shutdown( int sig )
 	 * SIGBREAK is generated when a user logs out.
 	 */
 
-#if 0
 #if HAVE_NT_SERVICE_MANAGER && SIGBREAK
 	if (is_NT_Service && sig == SIGBREAK)
-#ifdef NEW_LOGGING
-	    LDAP_LOG( CONNECTION, CRIT,
-		    "slap_sig_shutdown: SIGBREAK ignored.\n", 0, 0, 0 );
-#else
-	    Debug(LDAP_DEBUG_TRACE, "slap_sig_shutdown: SIGBREAK ignored.\n",
-		  0, 0, 0);
-#endif
+		;
 	else
 #endif
-#endif
 #ifdef SIGHUP
 	if (sig == SIGHUP && global_gentlehup && slapd_gentle_shutdown == 0)
 		slapd_gentle_shutdown = 1;
@@ -2113,8 +2128,8 @@ slap_sig_wake( int sig )
 }
 
 
-void slapd_add_internal(ber_socket_t s) {
-	slapd_add(s);
+void slapd_add_internal(ber_socket_t s, int isactive) {
+	slapd_add(s, isactive);
 }
 
 Listener ** slapd_get_listeners(void) {
