@@ -1,8 +1,27 @@
 /* dn.c - routines for dealing with distinguished names */
-/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.144.2.2 2003/10/06 14:57:29 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.144.2.4 2003/12/01 00:58:34 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
  */
 
 #include "portable.h"
@@ -314,7 +333,7 @@ LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 				 *	if value is empty, use empty_bv
 				 */
 				rc = ( *normf )(
-					0,
+					SLAP_MR_VALUE_OF_ASSERTION_SYNTAX,
 					ad->ad_type->sat_syntax,
 					mr,
 					ava->la_value.bv_len
@@ -329,8 +348,10 @@ LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 
 
 			if( bv.bv_val ) {
-				ber_memfree_x( ava->la_value.bv_val, ctx );
+				if ( ava->la_flags & LDAP_AVA_FREE_VALUE )
+					ber_memfree_x( ava->la_value.bv_val, ctx );
 				ava->la_value = bv;
+				ava->la_flags |= LDAP_AVA_FREE_VALUE;
 			}
 
 			if( do_sort ) AVA_Sort( rdn, iAVA );
@@ -882,7 +903,12 @@ int
 dnX509normalize( void *x509_name, struct berval *out )
 {
 	/* Invoke the LDAP library's converter with our schema-rewriter */
-	return ldap_X509dn2bv( x509_name, out, LDAPDN_rewrite, 0 );
+	int rc = ldap_X509dn2bv( x509_name, out, LDAPDN_rewrite, 0 );
+
+	Debug( LDAP_DEBUG_TRACE,
+		"dnX509Normalize: <%s>\n", out->bv_val, 0, 0 );
+
+	return rc;
 }
 
 /*
