@@ -1,10 +1,20 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/config.c,v 1.30.2.2 2003/10/06 14:57:32 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/config.c,v 1.30.2.4 2003/12/01 00:58:46 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * Portions Copyright 2003 Mark Benson.
+ * Portions Copyright 2002 John Morrissey.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +24,13 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).  Additional signficant contributors
+ * include:
+ *    John Morrissey
+ *    Mark Benson
+ */
 
 
 /*
@@ -523,10 +540,10 @@ parse_replica_line(
 	    fprintf( stderr, "slurpd no longer supports Kerberos.\n" );
 	    exit( EXIT_FAILURE );
 	    } else if ( !strcasecmp( val, SIMPLESTR )) {
-		ri->ri_bind_method = AUTH_SIMPLE;
+		ri->ri_bind_method = LDAP_AUTH_SIMPLE;
 		gots |= GOT_METHOD;
 	    } else if ( !strcasecmp( val, SASLSTR )) {
-		ri->ri_bind_method = AUTH_SASL;
+		ri->ri_bind_method = LDAP_AUTH_SASL;
 		gots |= GOT_METHOD;
 	    } else {
 		ri->ri_bind_method = -1;
@@ -575,14 +592,13 @@ parse_replica_line(
 	}
     }
     
-	if ( ri->ri_bind_method == AUTH_SASL) {
+	if ( ri->ri_bind_method == LDAP_AUTH_SASL) {
 		if ((gots & GOT_MECH) == 0) {
 			fprintf( stderr, "Error: \"replica\" line needs SASLmech flag in " );
 			fprintf( stderr, "slapd config file, line %d\n", lineno );
 			return -1;
 		}
-	}
-	else if ( gots != GOT_ALL ) {
+	} else if ( gots != GOT_ALL ) {
 		fprintf( stderr, "Error: Malformed \"replica\" line in slapd " );
 		fprintf( stderr, "config file, line %d\n", lineno );
 		return -1;
