@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.30.2.2 2003/10/13 01:57:04 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.30.2.4 2003/12/01 00:58:46 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2003 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +22,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 /* slurp.h - Standalone Ldap Update Replication Daemon (slurpd) */
 
@@ -65,9 +77,6 @@
 /* slurpd dump file - contents of rq struct are written here (debugging) */
 #define	SLURPD_DUMPFILE			LDAP_TMPDIR LDAP_DIRSEP "slurpd.dump"
 
-/* default srvtab file.  Can be overridden */
-#define	SRVTAB				"/etc/srvtab"
-
 /* Amount of time to sleep if no more work to do */
 #define	DEFAULT_NO_WORK_INTERVAL	3
 
@@ -85,11 +94,6 @@
 #define TLS_ON			1
 #define TLS_CRITICAL	2
 
-/* We support simple (plaintext password) and SASL authentication */
-#define	AUTH_SIMPLE	1
-#define	AUTH_KERBEROS	2
-#define	AUTH_SASL 3
-
 /* Rejection records are prefaced with this string */
 #define	ERROR_STR	"ERROR"
 
